@@ -37,6 +37,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <gdk/gdkx.h>
+
+#if DEBUG
+#define debug_printf(args...) do { g_print ("debug: " args); } while (0)
+#else
+#define debug_printf(args...) do { /* nothing */ } while (0)
+#endif
+
 
 /* Define local variables here */
 static gint posIV[4][16]; /* 0 - ypos, 1 - height, 2 - xpos, 3 - width */
@@ -94,6 +102,64 @@ void generate_animation_positions (struct tilda_window_ *tw)
     }
 }
 
+/* Shamelessly adapted (read: ripped off) from gdk_window_focus() and
+ * http://code.google.com/p/ttm/ trunk/src/window.c set_active()
+ *
+ * Also, more thanks to halfline and marnanel from irc.gnome.org #gnome
+ * for their help in figuring this out.
+ *
+ * Thank you. And boo to metacity, because they keep breaking us.
+ */
+
+/* This function will make sure that tilda window becomes active (gains
+ * the focus) when it is called.
+ *
+ * This has to be the worst possible way of making this work, but it was the
+ * only way to get metacity to play nicely. All the other WM's are so nice,
+ * why oh why does metacity hate us so?
+ */
+void tilda_window_set_active (tilda_window *tw)
+{
+    DEBUG_FUNCTION ("tilda_window_set_active");
+    DEBUG_ASSERT (tw != NULL);
+
+    Display *x11_display = GDK_WINDOW_XDISPLAY( tw->window->window );
+    Window *x11_window = GDK_WINDOW_XWINDOW( tw->window->window );
+    Window *x11_root_window = GDK_WINDOW_XWINDOW ( gtk_widget_get_root_window (tw->window) );
+    GdkScreen *screen = gtk_widget_get_screen (tw->window);
+
+    XEvent event;
+    long mask = SubstructureRedirectMask | SubstructureNotifyMask;
+
+    if (gdk_x11_screen_supports_net_wm_hint (screen,
+                                             gdk_atom_intern_static_string ("_NET_ACTIVE_WINDOW")))
+    {
+        event.xclient.type = ClientMessage;
+        event.xclient.serial = 0;
+        event.xclient.send_event = True;
+        event.xclient.display = x11_display;
+        event.xclient.window = x11_window;
+        event.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_ACTIVE_WINDOW");
+
+        event.xclient.format = 32;
+        event.xclient.data.l[0] = 2; /* pager */
+        event.xclient.data.l[1] = tomboy_keybinder_get_current_event_time(); /* timestamp */
+        event.xclient.data.l[2] = 0;
+        event.xclient.data.l[3] = 0;
+        event.xclient.data.l[4] = 0;
+
+        XSendEvent (x11_display, x11_root_window, False, mask, &event);
+    }
+    else
+    {
+        /* The WM doesn't support the EWMH standards. We'll print a warning and
+         * try this, though it probably won't work... */
+        g_printerr ("WARNING: Window manager (%s) does not support EWMH hints\n",
+                    gdk_x11_screen_get_window_manager_name (screen));
+        XRaiseWindow (x11_display, x11_window);
+    }
+}
+
 /* Process all pending GTK events, without returning to the GTK mainloop */
 static void process_all_pending_gtk_events ()
 {
@@ -119,10 +185,21 @@ void pull (struct tilda_window_ *tw, enum pull_state state)
         /* Keep things here just like they are. If you use gtk_window_present() here, you
          * will introduce some weird graphical glitches. Also, calling gtk_window_move()
          * before showing the window avoids yet more glitches. You should probably not use
-         * gtk_window_show_all() here, as it takes a long time to execute. */
+         * gtk_window_show_all() here, as it takes a long time to execute.
+         *
+         * Overriding the user time here seems to work a lot better than calling
+         * gtk_window_present_with_time() here, or at the end of the function. I have
+         * no idea why, they should do the same thing. */
+        gdk_x11_window_set_user_time (GTK_WIDGET(tw->window)->window,
+                                      tomboy_keybinder_get_current_event_time());
         gtk_window_move (GTK_WINDOW(tw->window), config_getint ("x_pos"), config_getint ("y_pos"));
         gtk_widget_show (GTK_WIDGET(tw->window));
 
+        /* Nasty code to make metacity behave. Starting at metacity-2.22 they "fixed" the
+         * focus stealing prevention to make the old _NET_WM_USER_TIME hack
+         * not work anymore. This is working for now... */
+        tilda_window_set_active (tw);
+
         /* The window should maintain its properties when it is merely hidden, but it does
          * not. If you delete the following call, the window will not remain visible
          * on all workspaces after pull()ing it up and down a number of times.
@@ -143,24 +220,8 @@ void pull (struct tilda_window_ *tw, enum pull_state state)
                 g_usleep (config_getint ("slide_sleep_usec"));
             }
         }
-        else
-        {
-            /* I know this seems redundant, but it really is necessary for Tilda to be
-             * pulled down in the correct position. */
-            gtk_window_move (GTK_WINDOW(tw->window), config_getint ("x_pos"), config_getint ("y_pos"));
-        }
 
-        /* Now that we're done with everything, we can call this. It overrides the
-         * focus stealing prevention that is present in openbox and others.
-         *
-         * WARNING: Don't move this up to the top, where the window is shown, or you
-         * will cause some strange glitches. :) */
-        gtk_window_present (GTK_WINDOW(tw->window));
-
-#if DEBUG
-        /* The window is definitely in the pulled down state now */
-        g_print ("pull(): MOVED DOWN\n");
-#endif
+        debug_printf ("pull(): MOVED DOWN\n");
         tw->current_state = DOWN;
     }
     else if (state != PULL_DOWN)
@@ -182,10 +243,7 @@ void pull (struct tilda_window_ *tw, enum pull_state state)
          * Case 2 - Animation off: Just hide the window */
         gtk_widget_hide (GTK_WIDGET(tw->window));
 
-#if DEBUG
-        /* The window is definitely in the UP state now */
-        g_print ("pull(): MOVED UP\n");
-#endif
+        debug_printf ("pull(): MOVED UP\n");
         tw->current_state = UP;
     }
 }
