@@ -337,8 +337,11 @@ static gboolean parse_cli (int argc, char *argv[])
     {
         g_print ("%s\n\n", TILDA_VERSION);
 
-        g_print ("Copyright (c) 2005,2006 Tristan Sloughter (sloutri@iit.edu)\n");
-        g_print ("Copyright (c) 2005-2007 Ira W. Snyder (tilda@irasnyder.com)\n\n");
+        g_print ("Copyright (c) 2005-2008 Tristan Sloughter (sloutri@iit.edu)\n");
+        g_print ("Copyright (c) 2005-2008 Ira W. Snyder (tilda@irasnyder.com)\n\n");
+
+        g_print ("General Information: http://launchpad.net/tilda\n");
+        g_print ("Bug Reports: http://bugs.launchpad.net/tilda\n\n");
 
         g_print ("This program comes with ABSOLUTELY NO WARRANTY.\n");
         g_print ("This is free software, and you are welcome to redistribute it\n");
