@@ -17,12 +17,13 @@
 #ifndef TILDA_WINDOW_H
 #define TILDA_WINDOW_H
 
+#include <tilda_window.h>
+#include <tilda_terminal.h>
+
+#include <glib.h>
 #include <gtk/gtk.h>
 
-#include "tilda_terminal.h"
-#include <confuse.h>
-
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
 typedef struct tilda_window_ tilda_window;
 
@@ -37,17 +38,24 @@ struct tilda_window_
     gchar *config_file;
     gboolean config_writing_disabled;
     gint instance;
+    gboolean have_argb_visual;
 
-    /* Stores all configuration */
-    cfg_t *tc;
+    /* This field MUST be set before calling pull()! */
+    enum tilda_positions { UP, DOWN } current_state;
 };
 
-void add_tab (tilda_window *tw);
-void add_tab_menu_call (gpointer data, guint callback_action, GtkWidget *w);
-void close_tab (gpointer data, guint callback_action, GtkWidget *w);
-gboolean init_tilda_window (tilda_window *tw, tilda_term *tt);
-void init_tilda_window_instance (tilda_window *tw);
+enum notebook_tab_positions { NB_TOP, NB_BOTTOM, NB_LEFT, NB_RIGHT };
 
-G_END_DECLS;
+gint tilda_window_add_tab (tilda_window *tw);
+gint tilda_window_close_tab (tilda_window *tw, gint tab_position);
+tilda_window *tilda_window_init (const gchar *config_file, const gint instance);
+gint tilda_window_free (tilda_window *tw);
+
+gint tilda_window_set_tab_position (tilda_window *tw, enum notebook_tab_positions pos);
+void tilda_window_close_current_tab (tilda_window *tw);
+
+#define TILDA_WINDOW(data) ((tilda_window *)(data))
+
+G_END_DECLS
 
 #endif
